"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var classification_1 = require("../classification");
var globalization_1 = require("../globalization");
var path_1 = require("path");
var xmldom_1 = require("xmldom");
var compiler = require("vue-template-compiler");
var fs_1 = require("fs");
var ts_simple_ast_1 = require("ts-simple-ast");
var typescript_1 = require("typescript");
var function_1 = require("../function");
var argumenter_1 = require("@joejukan/argumenter");
var web_kit_1 = require("@joejukan/web-kit");
var RENDER_NAME = "__render__";
var STATIC_RENDER_NAME = "__staticrender__";
var COMPONENTS_NAME = "__components__";
var REGEX_KEY = /\'[\w\-\%\_]+\'(?=\:)|\"[\w\-\%\_]+\"(?=\:)/gm;
var REGEX_BEGIN_QUOTE = /^(?:\"|\')/gm;
var REGEX_END_QUOTE = /(?:\"|\')$/gm;
var REGEX_RENDER = /(?:\'|\")\%\%\%RENDER\%\%\%(?:\'|\")/i;
var REGEX_STATIC_RENDER = /(?:\'|\")\%\%\%STATICRENDER\%\%\%(?:\'|\")/i;
var REGEX_COMPONENTS = /(?:\'|\")\%\%\%COMPONENTS\%\%\%(?:\'|\")/i;
var ASTClass = /** @class */ (function (_super) {
    __extends(ASTClass, _super);
    function ASTClass() {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        var _this = _super.call(this) || this;
        var argue = new argumenter_1.Argumenter(args);
        _this.path = argue.string;
        return _this;
    }
    Object.defineProperty(ASTClass.prototype, "name", {
        get: function () {
            var path = this.path;
            if (path) {
                var base = path_1.basename(path);
                var ext = path_1.extname(path);
                return base + "." + ext;
            }
        },
        enumerable: true,
        configurable: true
    });
    ASTClass.prototype.toImportDeclaration = function (dependency) {
        var declaration = {
            moduleSpecifier: dependency.relative(this.path)
        };
        if (dependency.defaulted) {
            declaration.defaultImport = dependency.symbol;
        }
        else {
            declaration.namedImports = [{ name: dependency.symbol }];
        }
        return declaration;
    };
    ASTClass.prototype.processScript = function () {
        var source = this.source = this.createSourceFile(this.name);
        source.insertText(0, this.typescript);
        var classes = source.getClasses();
        var template = this.template;
        for (var i = classes.length - 1; i >= 0; i--) {
            var cls = classes[i];
            // handle decorator
            var decorator = cls.getDecorator(function (dec) {
                var name = dec.getName();
                var isComp = /Component/i.test(name);
                return isComp;
            });
            if (decorator) {
                if (!decorator.isDecoratorFactory()) {
                    decorator.setIsDecoratorFactory(true);
                }
                var args = decorator.getArguments();
                var arg = (args.length > 0 ? args[0] : undefined);
                if (!arg) {
                    decorator.addArgument("{}");
                    arg = decorator.getArguments()[0];
                }
                var options = {};
                eval("options = " + arg.getText());
                if (!options.name) {
                    options.name = web_kit_1.kebab(cls.getName());
                }
                var renders = compiler.compileToFunctions(function_1.preCompile(this.template));
                var statics = renders.staticRenderFns;
                var staticStrings = "[ ";
                for (var j = 0; j < statics.length; j++) {
                    var s = statics[j];
                    if (j > 0)
                        staticStrings += ", ";
                    staticStrings += "function () { " + function_1.functionString(s) + " }";
                }
                staticStrings += " ]";
                var imports = new Array();
                // imports
                for (var k in globalization_1.dependencies) {
                    var dependency = globalization_1.dependencies[k];
                    var name_1 = dependency.name;
                    var symbol = dependency.symbol;
                    if (name_1 != options.name) {
                        var dom = new xmldom_1.DOMParser();
                        var doc = dom.parseFromString(template, 'text/xml').documentElement;
                        var elements = doc.getElementsByTagName(dependency.name);
                        if (elements.length > 0) {
                            imports.push(dependency);
                        }
                    }
                }
                // template variable
                source.insertVariableStatement(0, {
                    declarationKind: ts_simple_ast_1.VariableDeclarationKind.Const,
                    declarations: [
                        {
                            name: STATIC_RENDER_NAME,
                            type: "Array<Function>",
                            initializer: staticStrings
                        }
                    ]
                });
                source.insertFunction(0, {
                    name: RENDER_NAME,
                    bodyText: function_1.functionString(renders.render)
                });
                var componentString = "{\n";
                for (var j = 0; j < imports.length; j++) {
                    var imp = imports[j];
                    if (j > 0)
                        componentString += ",\n";
                    componentString += "'" + imp.name + "': " + imp.symbol;
                }
                componentString += "\n}";
                source.insertVariableStatement(0, {
                    declarationKind: ts_simple_ast_1.VariableDeclarationKind.Const,
                    declarations: [
                        {
                            name: COMPONENTS_NAME,
                            initializer: (imports.length > 0 ? componentString : "{}")
                        }
                    ]
                });
                for (var j = 0; j < imports.length; j++) {
                    var imp = imports[j];
                    source.insertImportDeclaration(0, this.toImportDeclaration(imp));
                }
                options.render = "%%%RENDER%%%";
                options.staticRenderFns = "%%%STATICRENDER%%%";
                options.components = "%%%COMPONENTS%%%";
                var optionString = JSON.stringify(options);
                optionString = optionString.replace(REGEX_KEY, function (key) {
                    return key.replace(REGEX_BEGIN_QUOTE, " ").replace(REGEX_END_QUOTE, "");
                });
                optionString = optionString.replace(REGEX_RENDER, " " + RENDER_NAME + " ");
                optionString = optionString.replace(REGEX_STATIC_RENDER, " " + STATIC_RENDER_NAME + " ");
                optionString = optionString.replace(REGEX_COMPONENTS, " " + COMPONENTS_NAME + " ");
                arg.replaceWithText(optionString);
            }
        }
        try {
            source.organizeImports();
        }
        catch (ex) {
        }
        this.typescript = source.getText();
    };
    ASTClass.prototype.load = function (content) {
        var sfc = this.sfc = compiler.parseComponent(content);
        this.typescript = sfc.script.content;
        this.template = "<div>" + sfc.template.content + "</div>";
        if (sfc.styles.length > 0) {
            this.style = sfc.styles[0].content;
        }
        this.processScript();
        this.javascript = typescript_1.transpile(this.typescript, {
            module: typescript_1.ModuleKind.CommonJS,
            moduleResolution: typescript_1.ModuleResolutionKind.NodeJs,
            emitDecoratorMetadata: true,
            experimentalDecorators: true,
            allowSyntheticDefaultImports: true,
            noImplicitUseStrict: true
        });
    };
    ASTClass.prototype.pitch = function (path) {
        var content = fs_1.readFileSync(path, 'utf-8');
        content = "<source>" + content + "</source>";
        var source = this.createSourceFile(web_kit_1.uuid() + ".ts");
        source.insertText(0, content);
        var classes = source.getClasses();
        for (var i = 0; i < classes.length; i++) {
            var cls = classes[i];
            var decorators = cls.getDecorators();
            for (var j = 0; j < decorators.length; j++) {
                var decorator = decorators[j];
                if (decorator.getName() === "Component") {
                    var args = decorator.getArguments() || [];
                    var opts = (args.length > 0 ? args[0].getText() : "{}");
                    var options = {};
                    eval("options = " + opts);
                    var name_2 = undefined;
                    var symbol = cls.getName();
                    if (options.name) {
                        name_2 = options.name;
                    }
                    else {
                        name_2 = web_kit_1.kebab(symbol);
                    }
                    globalization_1.dependencies[name_2] = new classification_1.DependencyClass(name_2, symbol, path, cls.isDefaultExport());
                    return;
                }
            }
        }
    };
    ASTClass.prototype.inject = function (content) {
        var source = this.createSourceFile(web_kit_1.uuid() + ".ts");
        source.replaceWithText(content);
        if (!source)
            return;
        var finder = new classification_1.ASTFinder(source);
        var declaration = finder.findVariableDeclaration('CombinedVueInstance');
        if (!declaration)
            return;
        var lastChild = declaration.getLastChild();
        var expression = declaration.getFirstChildByKind(ts_simple_ast_1.SyntaxKind.NewExpression);
        if (!expression)
            return;
        var args = expression.getArguments();
        if (!args || args.length != 1)
            return;
        var arg = args[0];
        var object = arg.getFirstChildByKind(ts_simple_ast_1.SyntaxKind.SyntaxList);
        if (!object)
            return;
        var pairs = object.getChildren();
        if (!pairs)
            return;
        var components;
        pairs.forEach(function (pair) {
            var key = pair.getFirstChildByKind(ts_simple_ast_1.SyntaxKind.Identifier);
            var kind = pair.getKind();
            if (key && key.getText() === 'components') {
                if (kind === ts_simple_ast_1.SyntaxKind.ShorthandPropertyAssignment) {
                    pair.replaceWithText('components: {...components}');
                }
                else if (kind === ts_simple_ast_1.SyntaxKind.PropertyAssignment) {
                }
                else {
                    pair.replaceWithText('components: {}');
                }
                components = pair.getFirstChildByKind(ts_simple_ast_1.SyntaxKind.ObjectLiteralExpression);
            }
        });
        if (!components) {
            if (object.getLastChild().getKind() !== ts_simple_ast_1.SyntaxKind.CommaToken) {
                object.addChildText(',');
            }
            components = object.addChildText('components: {}')[0].getFirstChildByKind(ts_simple_ast_1.SyntaxKind.ObjectLiteralExpression);
        }
        for (var k in globalization_1.dependencies) {
            var dependency = globalization_1.dependencies[k];
            source.addImportDeclaration(this.toImportDeclaration(dependency));
            components.addPropertyAssignment({ name: "'" + dependency.name + "'", initializer: dependency.symbol });
        }
        source.organizeImports();
        console.log(source.getText());
        this.typescript = source.getText();
    };
    return ASTClass;
}(ts_simple_ast_1.default));
exports.ASTClass = ASTClass;
//# sourceMappingURL=ast.class.js.map